/****************************************************************************

Copyright 2007 Virginia Polytechnic Institute and State University

This file is part of the OSSIE dsrc_data_generator.

OSSIE dsrc_data_generator is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

OSSIE dsrc_data_generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with OSSIE dsrc_data_generator; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

****************************************************************************/


#include <string>
#include <iostream>
#include "dsrc_data_generator.h"

dsrc_data_generator_i::dsrc_data_generator_i(const char *uuid, omni_condition *condition) :
	Resource_impl(uuid), component_running(condition)
{
	dataOut_0 = new standardInterfaces_i::realChar_u("dataOut");

	//Create the thread for the writer's processing function
	processing_thread = new omni_thread(Run, (void *) this);

	//Start the thread containing the writer's processing function
	processing_thread->start();

}

dsrc_data_generator_i::~dsrc_data_generator_i(void)
{
	delete dataOut_0;
}

// Static function for omni thread
void dsrc_data_generator_i::Run( void * data )
{
	((dsrc_data_generator_i*)data)->ProcessData();
}

CORBA::Object_ptr dsrc_data_generator_i::getPort( const char* portName ) throw (
	CORBA::SystemException, CF::PortSupplier::UnknownPort)
{
	DEBUG(3, dsrc_data_generator, "getPort() invoked with " << portName)

	CORBA::Object_var p;

	p = dataOut_0->getPort(portName);

	if (!CORBA::is_nil(p))
		return p._retn();

	/*exception*/
	throw CF::PortSupplier::UnknownPort();
}

void dsrc_data_generator_i::start() throw (CORBA::SystemException,
	CF::Resource::StartError)
{
	DEBUG(3, dsrc_data_generator, "start() invoked")
	omni_mutex_lock l(processing_mutex);
	if (false == thread_started){
		thread_started = true;    //initialize
		//Create the thread for the writer's processing function
		processing_thread = new omni_thread(Run, (void *) this);

		//Start the thread containing the writer's processing function
		processing_thread->start();
	}
}

void dsrc_data_generator_i::stop() throw (CORBA::SystemException, CF::Resource::StopError)
{
	DEBUG(3, dsrc_data_generator, "stop() invoked")
	omni_mutex_lock l(processing_mutex);
	thread_started=false;
}

void dsrc_data_generator_i::releaseObject() throw (CORBA::SystemException,
	CF::LifeCycle::ReleaseError)
{
	DEBUG(3, dsrc_data_generator, "releaseObject() invoked")

	component_running->signal();
}

void dsrc_data_generator_i::initialize() throw (CF::LifeCycle::InitializeError,
	CORBA::SystemException)
{
	DEBUG(3, dsrc_data_generator, "initialize() invoked")
}

void dsrc_data_generator_i::configure(const CF::Properties& props)
throw (CORBA::SystemException,
	CF::PropertySet::InvalidConfiguration,
	CF::PropertySet::PartialConfiguration)
{
	DEBUG(3, dsrc_data_generator, "configure() invoked")

	std::cout << "props length : " << props.length() << std::endl;

	for (unsigned int i = 0; i <props.length(); i++)
	{
		std::cout << "Property id : " << props[i].id << std::endl;

		if (strcmp(props[i].id, "DCE:3f6def6c-1395-11de-93ca-000c29aafdf2") == 0)
		{
			CORBA::Short simple_temp;
			props[i].value >>= simple_temp;
			simple_0_value = simple_temp;
		}

		if (strcmp(props[i].id, "DCE:edd2992e-3a17-11de-93cb-000c29aafdf2") == 0)
		{
			CORBA::Short simple_temp;
			props[i].value >>= simple_temp;
			simple_1_value = simple_temp;
		}

	}
}

const unsigned int SIN_LENGTH = 68;
const unsigned int PREAMBLE_LENGTH = 4992; // 4992 * 2 micro seconds = about 10 milli seconds

const unsigned int BST_LENGTH = 68;
const unsigned char BST[BST_LENGTH] = {
	0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
	1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};


void dsrc_data_generator_i::ProcessData()
{
	DEBUG(3, dsrc_data_generator, "ProcessData() invoked")

	PortTypes::CharSequence I_out_0;


	while(continue_processing())
	{
		short delay = (short) simple_0_value;
		short type_of_output = (short) simple_1_value;

		if (type_of_output == 1) {
			I_out_0.length(SIN_LENGTH); // length of output
			for (unsigned int i=0; i<SIN_LENGTH; i++)
			{
				I_out_0[i] = 0; // Binary "0"s equals a sine wave
			}
			dataOut_0->pushPacket(I_out_0);
		}

		else {// Then the data is a BTS with a preabmle
			I_out_0.length(PREAMBLE_LENGTH + BST_LENGTH); // length of output
			for (unsigned int i=0; i<PREAMBLE_LENGTH; i++)
			{
				I_out_0[i] = 1; // The preable consists of binary "1"s
			}
			for (unsigned int i=0; i<SIN_LENGTH; i++)
			{
				I_out_0[PREAMBLE_LENGTH + i] = BST[i];
			}
			dataOut_0->pushPacket(I_out_0);
			usleep(delay*1000); // Sleep given number of milli seconds
		}
	}
}

bool dsrc_data_generator_i::continue_processing(void)
{
	omni_mutex_lock l(processing_mutex);
	return thread_started;
}
