#!/usr/bin/env python
#-*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: DSRC RSU Receiver
# Author: Einar Thorsrud
# Description: Receiver for the DSRC RSU using the USRP.
##################################################

from gnuradio import gr
from gnuradio import eng_notation
from gnuradio.eng_option import eng_option
from gnuradio import usrp
from usrpm import usrp_dbid
from grc_gnuradio import usrp as grc_usrp
from optparse import OptionParser
import time
from gnuradio import dsrc

class top_block(gr.top_block):

	def __init__(self):
		gr.top_block.__init__(self, "DSRC RSU Receiver path")

		##################################################
		# Options
		##################################################
		parser = OptionParser(option_class=eng_option)

		parser.add_option("-o", "--output-file", type="string", default="binary_nrz_data.dat",
			help="specify output-data-filename [default=%default]")		
						
		parser.add_option("-d", "--duration", type="eng_float", default=0,
			help="specify the duration in milliseconds [default=%default]")
						
		parser.add_option("-f", "--subcarrier-freq", type="eng_float", default=0,
			help="specify the subcarrier frequency in Hertz [default=%default]")
						
		parser.add_option("-s", "--source", type="string", default="usrp",
			help="specify the source (\"usrp\" or \"file\") [default=%default]")
						
		parser.add_option('-v', '--verbose', action="store_true", default=False,
			help="verbose output")

		parser.add_option('-r', '--repeat', action="store_true", default=False,
			help="repeat input file sequence (if used)")

		(options, args) = parser.parse_args ()
		if len(args) != 0:
			parser.print_help()
			raise SystemExit, 1

		self.output_file = options.output_file
		self.duration_in_ms = options.duration
		self.frequency = options.subcarrier_freq
		self.source = options.source
		self.verbose = options.verbose
		self.repeat = options.repeat

		##################################################
		# Variables and constants
		##################################################
		self.samp_per_symb = 16 # Changes the amount of data over USB
		self.gain = 1.0 # Gain in the USRP
		self.symbol_per_second = 250e3 # Symbol rate equals the bit rate
		self.usrp_samples_per_second = 64e6
		self.input_rate = int(self.samp_per_symb * self.symbol_per_second)
		self.decimation_rate = int(self.usrp_samples_per_second / self.input_rate)
		
		# Variable for the MPSK-receiver synchronization block:
		M = 2 # The modulation order of the MPSK modulation
		costas_theta = 0
		costas_alpha = 0.01
		costas_beta = 0.000025
		costas_fmin = -0.00393 # Min normalized frequency
		costas_fmax = 0.00393 # Max normalized frequency
		m_and_m_mu = 0.5 # Start value [0,1]
		m_and_m_gain_mu = 0.05 # Gain parameter of the M&M signal
		m_and_m_omega = self.samp_per_symb
		m_and_m_gain_omega = (m_and_m_omega * m_and_m_omega) / 4 # Loop gain (Beta)
		m_and_m_omega_rel = 0.005
			
		self.lowpass_coeff = gr.firdes.low_pass (
			1.0,				# gain
			self.input_rate,	# sampling rate
			100e3,				# low pass cutoff freq
			100e3,				# width of trans. band
			gr.firdes.WIN_HAMMING) # Window

		
		##################################################
		# Generic blocks (Non USRP)
		##################################################
		
		# synchronization block (Costas and mM&M)
		self.mpsk_receiver = gr.mpsk_receiver_cc(
			M, 
			costas_theta,
			costas_alpha,
			costas_beta,
			costas_fmin,
			costas_fmax,
			m_and_m_mu,
			m_and_m_gain_mu,
			m_and_m_omega,
			m_and_m_gain_omega,
			m_and_m_omega_rel)
		
		# Extract data by hard decision:
		self.complex_to_real = gr.complex_to_real(1) # Remove imaginary part
		self.binary_slicer = gr.binary_slicer_fb() # Decision device
		
		# NRZI to NRZ (differential decoding)
		self.nrzi_to_nrz = dsrc.nrzi_to_nrz_bb (1) # Initial state 1 or 0
		
		# Frequency translating low pass FIR filter for channel selection
		# self.channel_filter = gr.fir_filter_ccf(1, self.lowpass_coeff)
		self.channel_filter = \
			gr.freq_xlating_fir_filter_ccf (
				1, # 1 = no decimation
				self.lowpass_coeff,	# Channel filter coef
				self.frequency,		# Station frequency
				self.input_rate)		# Input sampling rate


		# File sink for the decoded data
		self.file_sink_decoded_data = gr.file_sink(gr.sizeof_char, self.output_file)


		##################################################
		# USRP blocks and connections
		##################################################
		if self.source.lower() == 'usrp':
			self.usrp_source = usrp.source_c(0, self.decimation_rate)
			self.subdev_spec = (0,2) # Side A, sub device A and B combined
			self.usrp_source.set_mux(usrp.determine_rx_mux_value(self.usrp_source, self.subdev_spec))
			self.subdev = usrp.selected_subdev(self.usrp_source, self.subdev_spec)
			self.usrp_source.set_rx_freq(0, 0.0) # Set frequency 0.0 Hz on board number 0
			self.subdev.set_gain(self.gain)
		
			self.connect((self.usrp_source, 0), (self.channel_filter, 0))
		
		
		##################################################
		# File source blocks and connections
		##################################################
		else: #if self.source.lower() != 'usrp':
			# Block to read file with complex samples when not using the USRP
			# False/True decided whether to repeat indefinitely or not
			self.file_source = gr.file_source(gr.sizeof_gr_complex*1, "raw_input_samples.dat", self.repeat)
			
			self.connect((self.file_source, 0), (self.channel_filter, 0))
		
		
		##################################################
		# Verbose blocks and connections (log to files)
		##################################################
		if self.verbose:
			self.file_sink_samples = gr.file_sink(gr.sizeof_gr_complex, "complex_raw_samples.dat")
			self.file_sink_filtered_samples = gr.file_sink(gr.sizeof_gr_complex, "complex_filtered_samples.dat")
			self.file_sink_symbols = gr.file_sink(gr.sizeof_gr_complex, "complex_symbols.dat")
			self.file_sink_data = gr.file_sink(gr.sizeof_char, "binary_nrzi_data.dat")

			
			if self.source.lower() == 'usrp':
				self.connect((self.usrp_source, 0), (self.file_sink_samples, 0))
			else:
				self.connect((self.file_source, 0), (self.file_sink_samples, 0))
			
			self.connect((self.channel_filter, 0), (self.file_sink_filtered_samples, 0))
			
			# For writing synchronized symbols to file
			self.connect((self.mpsk_receiver, 0), (self.file_sink_symbols, 0))
			
			# For writing binary data to file
			self.connect((self.binary_slicer, 0), (self.file_sink_data, 0))
			
	
	
		##################################################
		# Other connections
		##################################################
		# Connect file_source to synchronization block:
		self.connect((self.channel_filter, 0), (self.mpsk_receiver, 0))
		
		# Connect symbols to binary data
		self.connect((self.mpsk_receiver, 0), (self.complex_to_real, 0))
		self.connect((self.complex_to_real, 0), (self.binary_slicer, 0))
		
		# Convert NRZI to NRZ
		self.connect((self.binary_slicer, 0), (self.nrzi_to_nrz, 0))
		
		# Write binary data to file
		self.connect((self.nrzi_to_nrz, 0), (self.file_sink_decoded_data, 0))


if __name__ == '__main__':

	# Starting flowgraph
	tb = top_block()
	
	if tb.duration_in_ms > 0: # Duration has been specified
		print "Receiving for " + str(tb.duration_in_ms) + " milliseconds..."
		tb.start()
		# Wait specified ammount of time befor terminating the flowgraph
		time.sleep(tb.duration_in_ms/1000.0)
		tb.stop()
	
	elif tb.source.lower() != 'usrp': # File source
		tb.run()

	else: # USRP source, receving untill terminated
		tb.start()
		raw_input('Press Enter to quit: ')
		tb.stop()

	print "\nFinished! Output data is stored in: " + tb.output_file
