#!/usr/bin/env python
#-*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: DSRC RSU Transmitter
# Author: Einar Thorsrud
# Description: Transmitter for the DSRC RSU using the USRP.
##################################################

from gnuradio import gr
from gnuradio import eng_notation
from gnuradio.eng_option import eng_option
from grc_gnuradio import usrp as grc_usrp
from gnuradio import dsrc
from optparse import OptionParser
import time

class top_block(gr.top_block):

	def __init__(self):
		gr.top_block.__init__(self, "DSRC RSU Transmitter")

		##################################################
		# Options
		##################################################
		parser = OptionParser(option_class=eng_option)

		parser.add_option("-i", "--input-file", type="string", default="binary_nrz_data.dat",
			help="specify input-data-filename [default=%default]")
		
		parser.add_option("-o", "--output-file", type="string", default="output_samples.dat",
			help="specify output-data-filename [default=%default]")
						
		parser.add_option("-s", "--sink", type="string", default="usrp",
			help="specify the sink (\"usrp\" or \"file\") [default=%default]")
						
		parser.add_option('-r', '--repeat', action="store_true", default=False,
			help="repeat input file or BST sequence")
						
		parser.add_option('-c', '--carrier', action="store_true", default=False,
			help="pure sine wave")
						
		parser.add_option('-b', '--bst', action="store_true", default=False,
			help="send a BST sequence every 10 millisecond")
		
		(options, args) = parser.parse_args ()
		if len(args) != 0:
			parser.print_help()
			raise SystemExit, 1

		self.input_file = options.input_file
		self.output_file = options.output_file
		self.sink = options.sink
		self.repeat = options.repeat
		self.carrier = options.carrier
		self.bst = options.bst

		##################################################
		# Variables
		##################################################
		self.frequency = 0
		self.gain = 1.0 # USRP transmitter gain
		self.bit_per_second = 500e3 # For MDR this is 500 kbit/s
		self.usrp_samples_per_second = 128e6 # The USRP DAC is 128 MS/s
		self.frequency = 0
		
		# 25000 max and -300 was OK with a deffect RF-frontend
		self.v_min = 25000	# within the range -32768/+32767 
		self.v_max = 1000 # within the range -32768/+32767
		self.phase = 1 # +1 or -1
		self.pulse_shaper_interpolation = 16

		# Variable calculated based on the constants above 
		self.interpolation_rate = 2 * self.usrp_samples_per_second / (self.bit_per_second * self.pulse_shaper_interpolation)

		##################################################
		# Generic blocks (Non USRP) and connections
		##################################################
		self.pulse_shaper = dsrc.pulse_shaper_bs (self.v_min, self.v_max, self.phase, self.pulse_shaper_interpolation)
		
		##################################################
		# Generic blocks (Non USRP) and connections
		##################################################
		if self.carrier == True:
			self.source = gr.vector_source_b((0, 0, 0, 0), True) # Sine at 500 kHz
		
		elif self.bst == True:
			self.preamble = (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)

			self.bst_sequence = (
				0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, \
				0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, \
				0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, \
				0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, \
				0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 )
				
			# Sending the preamble 312 first to wake up the OBU, then a BST
			self.source = gr.vector_source_b(self.preamble * 312 + self.bst_sequence + self.preamble, self.repeat) 
		
		else:
			self.source = gr.file_source(gr.sizeof_char*1, self.input_file, self.repeat)

		##################################################
		# USRP sink blocks
		##################################################
		if self.sink.lower() == 'usrp':
			self.sink = grc_usrp.simple_sink_s(which=0, side='A')
			self.sink.set_interp_rate(self.interpolation_rate)
			self.sink.set_frequency(self.frequency, verbose=False)
			self.sink.set_gain(self.gain)
		
		##################################################
		# File sink block
		##################################################		
		else:
			self.sink = gr.file_sink(gr.sizeof_short, self.output_file)

		##################################################
		# Connections
		##################################################
		self.connect((self.source, 0), (self.pulse_shaper, 0))
		self.connect((self.pulse_shaper, 0), (self.sink, 0))


if __name__ == '__main__':
	
	tb = top_block()
	
	if tb.bst:
		if tb.repeat:
			tb.start()
			raw_input('Press Enter to quit: ')
			tb.stop()
		else:
			tb.run()

	
	elif tb.repeat:
		tb.start()
		raw_input('Press Enter to quit: ')
		tb.stop()
		
	else:
		tb.run()
